abstract class RemoteResource<T> {
  RemoteResource._();

  factory RemoteResource.loading() {
    return LoadingRemoteResource();
  }

  factory RemoteResource.error(String errorMessage) {
    return ErrorRemoteResource(errorMessage);
  }

  factory RemoteResource.finished(T value) {
    return FinishedRemoteResource(value);
  }

  RemoteResource<T> mapIfFinished(T Function(T) mapper) {
    if (this is FinishedRemoteResource) {
      return RemoteResource.finished(mapper(asFinished().value));
    }
    return this;
  }

  FinishedRemoteResource<T> asFinished() {
    return this as FinishedRemoteResource<T>;
  }

  ErrorRemoteResource<T> asError() {
    return this as ErrorRemoteResource<T>;
  }
}

class ErrorRemoteResource<T> extends RemoteResource<T> {
  final String errorMessage;

  ErrorRemoteResource(this.errorMessage) : super._();
}

class LoadingRemoteResource<T> extends RemoteResource<T> {
  LoadingRemoteResource() : super._();
}

class FinishedRemoteResource<T> extends RemoteResource<T> {
  final T value;
  FinishedRemoteResource(this.value) : super._();
}
