import 'package:flutter/widgets.dart';
import 'package:thesis_shop/models/product.dart';
import 'package:thesis_shop/models/remote_resource.dart';
import 'package:thesis_shop/service/product_service.dart';
import 'package:thesis_shop/stores/user_store.dart';

class ProductStoreImplementation extends StatefulWidget {
  final Widget child;
  final ProductService productService;
  const ProductStoreImplementation({
    Key? key,
    required this.child,
    required this.productService,
  }) : super(key: key);

  @override
  _ProductStoreImplementationState createState() =>
      _ProductStoreImplementationState();
}

class _ProductStoreImplementationState
    extends State<ProductStoreImplementation> {
  RemoteResource<List<Product>> products = RemoteResource.loading();

  @override
  void initState() {
    super.initState();
    loadProducts().catchError(
      (err) => setState(
        () {
          products = RemoteResource.error(err.toString());
        },
      ),
    );
  }

  Future<void> loadProducts() async {
    final remoteProducts = await widget.productService.fetchProducts();
    setState(() {
      products = RemoteResource.finished(remoteProducts);
    });
  }

  @override
  Widget build(BuildContext context) {
    final userStore = UserStore.of(context);
    if (userStore.isSignedIn) {
      return ProductStore(
        products: products.mapIfFinished(
          (products) => products
              .map(
                (product) => Product(
                  title: product.title,
                  price: product.price - product.price * 0.2,
                ),
              )
              .toList(),
        ),
        child: widget.child,
      );
    }
    return ProductStore(products: products, child: widget.child);
  }
}

class ProductStore extends InheritedWidget {
  const ProductStore({
    Key? key,
    required this.products,
    required Widget child,
  }) : super(key: key, child: child);

  final RemoteResource<List<Product>> products;
  List<Product> get mustProducts => products.asFinished().value;
  List<Product>? get productsOrNull {
    if (products is FinishedRemoteResource) {
      return mustProducts;
    }
    return null;
  }

  static ProductStore of(BuildContext context) {
    final ProductStore? result =
        context.dependOnInheritedWidgetOfExactType<ProductStore>();
    assert(result != null, 'No ProductStore found in context');
    return result!;
  }

  @override
  bool updateShouldNotify(ProductStore oldWidget) {
    return oldWidget.products != products;
  }
}
